@@ -27,7 +27,12 @@ class Importer(object):
         return f'{self.name_get()}@{self.args.domain}'
 
     def members_get(self):
-        return self.mailman.user.values()
+        return [v for k, v in self.mailman.user.items()
+                if v.get('user_options', 0) & Mailman.USER_OPTIONS_MODERATED == 0]
+
+    def moderated_members_get(self):
+        return [v for k, v in self.mailman.user.items()
+                if v.get('user_options', 0) & Mailman.USER_OPTIONS_MODERATED != 0]
 
     def owners_get(self):
         return [u for u in self.mailman.user.values() if u.get('owner')]
@@ -49,6 +54,14 @@ class Importer(object):
     def group_members_name(self):
         return Discourse.enforce_group_name_length(self.category2members_group(self.name_get()))
 
+    @staticmethod
+    def category2moderated_members_group(name):
+        return f'{name}-mm'
+
+    def group_moderated_members_name(self):
+        return Discourse.enforce_group_name_length(
+            self.category2moderated_members_group(self.name_get()))
+
     def user_create(self, email, name):
         username = None
         if name:
@@ -182,6 +195,7 @@ class Importer(object):
             kwargs.update({
                 'email_in_allow_strangers': 'true',
                 f'permissions[{self.group_members_name()}]': create_reply_see,
+                f'permissions[{self.group_moderated_members_name()}]': see,
             })
         elif self.moderation_get() == 'ignore' and archive == 'public':
             kwargs.update({
@@ -193,6 +207,7 @@ class Importer(object):
             kwargs.update({
                 'email_in_allow_strangers': 'false',
                 f'permissions[{self.group_members_name()}]': create_reply_see,
+                f'permissions[{self.group_moderated_members_name()}]': see,
             })
         elif self.moderation_get() == 'approval' and archive == 'public':
             kwargs.update({
@@ -205,6 +220,7 @@ class Importer(object):
             kwargs.update({
                 'email_in_allow_strangers': 'true',
                 f'permissions[{self.group_members_name()}]': create_reply_see,
+                f'permissions[{self.group_moderated_members_name()}]': create_reply_see,
                 'custom_fields[require_topic_approval]': 'true',
                 'custom_fields[require_reply_approval]': 'true',
                 'reviewable_by_group_name': self.group_moderators_name(),
@@ -216,6 +232,11 @@ class Importer(object):
         return kwargs
 
     def group_moderators_settings(self):
+        group_visibility_level = {
+            Mailman.ROSTER_PUBLIC: Discourse.GROUP_VISIBILITY_LEVEL_EVERYONE,
+            Mailman.ROSTER_MEMBERS: Discourse.GROUP_VISIBILITY_LEVEL_OWNERS_AND_MEMBERS,
+            Mailman.ROSTER_ADMINS: Discourse.GROUP_VISIBILITY_LEVEL_OWNERS_AND_MEMBERS,
+        }[self.mailman.info['private_roster']]
         members_visibility_level = {
             Mailman.ROSTER_PUBLIC: Discourse.GROUP_MEMBERS_VISIBILITY_LEVEL_EVERYONE,
             Mailman.ROSTER_MEMBERS: Discourse.GROUP_MEMBERS_VISIBILITY_LEVEL_MEMBERS,
@@ -223,13 +244,13 @@ class Importer(object):
         }[self.mailman.info['private_roster']]
         return {
             'group[full_name]': self.category2moderators_group(self.name_get()),
-            'group[visibility_level]': '0',
+            'group[visibility_level]': group_visibility_level,
             'group[members_visibility_level]': members_visibility_level,
             'group[public_admission]': 'false',
             'group[allow_membership_requests]': 'false',
         }
 
-    def group_members_settings(self):
+    def _group_members_settings(self):
         private_roster = self.mailman.info['private_roster']
         subscribe_policy = self.mailman.info['subscribe_policy']
         if subscribe_policy in (Mailman.SUBSCRIBE_POLICY_OPEN,
@@ -286,9 +307,18 @@ class Importer(object):
             },
         }
         kwargs = converter[(private_roster, policy)]
+        return kwargs
+
+    def group_members_settings(self):
+        kwargs = self._group_members_settings()
         kwargs['group[full_name]'] = self.category2members_group(self.name_get())
         return kwargs
 
+    def group_moderated_members_settings(self):
+        kwargs = self._group_members_settings()
+        kwargs['group[full_name]'] = self.category2moderated_members_group(self.name_get())
+        return kwargs
+
     def settings(self):
         kwargs = {
             'disallow_reply_by_email_after_days': '0',
@@ -346,6 +376,17 @@ class Importer(object):
             (_, after['group-members']) = self.discourse.group_create(
                 self.group_members_name(), **self.group_members_settings())
 
+            logger.info('Populating the moderated members group')
+            (before['group-moderated-members'], _) = self.discourse.group_create(
+                self.group_moderated_members_name())
+            (before['group-moderated-members-list'],
+             after['group-moderated-members-list']) = self.group_members_add(
+                 self.group_moderated_members_name(),
+                 self.moderators_get() + self.moderated_members_get(),
+                 self.owners_get())
+            (_, after['group-moderated-members']) = self.discourse.group_create(
+                self.group_moderated_members_name(), **self.group_moderated_members_settings())
+
         (_, after['category']) = self.discourse.category_set(
             self.name_get(), **self.category_settings())
 
