@@ -27,4 +27,5 @@ def test_options(version, tmpdir):
                               url=url,
                               mailman_encoding='UTF-8',
                               preserve_existing_user_preferences=False,
-                              force_private=False)
+                              force_private=False,
+                              skip_ban_list=False)
