@@ -22,26 +22,32 @@ def test_group_moderators_settings(test_options):
     i = importer.Importer(test_options)
     i.mailman.info['private_roster'] = 2
     s = i.group_moderators_settings()
-    assert s['group[members_visibility_level]'] == '3'
+    assert (s['group[members_visibility_level]'] ==
+            Discourse.GROUP_MEMBERS_VISIBILITY_LEVEL_GROUP_OWNERS_AND_STAFF)
+    assert s['group[visibility_level]'] == Discourse.GROUP_VISIBILITY_LEVEL_OWNERS_AND_MEMBERS
     assert s['group[full_name]'] == 'listname-mo'
 
 
-def test_group_members_settings(test_options):
+@pytest.mark.parametrize("fun,name", [
+    (importer.Importer.group_members_settings, 'listname-me'),
+    (importer.Importer.group_moderated_members_settings, 'listname-mm'),
+])
+def test_group_members_settings(test_options, fun, name):
     m_everyone = Discourse.GROUP_MEMBERS_VISIBILITY_LEVEL_EVERYONE
     g_everyone = Discourse.GROUP_VISIBILITY_LEVEL_EVERYONE
 
     incoming = (Mailman.ROSTER_PUBLIC, Mailman.SUBSCRIBE_POLICY_OPEN)
     expected = {
-         'group[full_name]': 'listname-me',
-         'group[public_admission]': 'true',
-         'group[allow_membership_requests]': 'false',
-         'group[visibility_level]': g_everyone,
-         'group[members_visibility_level]': m_everyone,
+        'group[public_admission]': 'true',
+        'group[full_name]': name,
+        'group[allow_membership_requests]': 'false',
+        'group[visibility_level]': g_everyone,
+        'group[members_visibility_level]': m_everyone,
      }
 
     i = importer.Importer(test_options)
     (i.mailman.info['private_roster'], i.mailman.info['subscribe_policy']) = incoming
-    assert i.group_members_settings() == expected
+    assert fun(i) == expected
 
 
 def test_users_add_invalid(test_options):
@@ -187,6 +193,7 @@ def test_importer_moderation_no(test_options):
         assert list(before0['group-moderators-list'].keys()) == [owner, moderator]
         assert all(v is None for v in before0['group-moderators-list'].values())
         assert 'group-members-list' not in before0
+        assert 'group-moderated-members-list' not in before0
         assert before0['group-moderators'] is None
         assert list(before0['members'].keys()) == [owner, moderator]
         assert 'email_in' in before0['settings']
@@ -272,12 +279,68 @@ def test_importer_moderation_approval(test_options):
     finally:
         i.discourse.group_delete(i.group_moderators_name())
         i.discourse.group_delete(i.group_members_name())
+        i.discourse.group_delete(i.group_moderated_members_name())
         i.discourse.category_delete(i.name_get())
         i.discourse.user_delete(moderator)
         i.discourse.user_delete(owner)
         i.discourse.user_delete(member)
 
 
+def test_importer_moderated_members(test_options):
+    i = importer.Importer(test_options)
+    owner = 'owner@example.com'
+    moderator = 'moderator@example.com'
+    member = 'member@example.com'
+    moderated_member = 'moderated_member@example.com'
+    try:
+        c = i.mailman.load_pickle()
+        c['default_member_moderation'] = True
+        c['member_moderation_action'] = Mailman.MODERATION_HOLD
+        mailman_add(c, owner, owner=True)
+        mailman_add(c, moderator, moderator=True)
+        mailman_add(c, member)
+        mailman_add(c, moderated_member, user_options=Mailman.USER_OPTIONS_MODERATED)
+        mailman_write_config(test_options.mailman_config, c)
+        i.mailman.load()
+
+        before0, after0 = i.importer()
+        assert before0['category'] is None
+        assert list(before0['group-moderators-list'].keys()) == [owner, moderator]
+        assert all(v is None for v in before0['group-moderators-list'].values())
+        assert list(before0['group-members-list'].keys()) == [owner, moderator, member]
+        assert all(v is None for v in before0['group-members-list'].values())
+        assert list(before0['group-moderated-members-list'].keys()) == [
+            owner, moderator, moderated_member]
+        assert all(v is None for v in before0['group-members-list'].values())
+        assert before0['group-moderators'] is None
+        assert list(before0['members'].keys()) == [owner, moderator, member, moderated_member]
+        assert 'email_in' in before0['settings']
+
+        assert after0['category']['name'] == test_options.list
+        assert after0['settings']['email_in'] == 'true'
+        assert after0['group-moderators']['group[name]'].startswith(test_options.list)
+        for who in (owner, member, moderator):
+            assert who in after0['group-members-list']
+        for who in (owner, moderated_member, moderator):
+            assert who in after0['group-moderated-members-list']
+        for who in (owner, moderator):
+            assert who in after0['group-moderators-list']
+        assert member not in after0['group-moderators-list']
+
+        before1, after1 = i.importer()
+        assert before1 == after0
+        assert before1 == after1
+    finally:
+        i.discourse.group_delete(i.group_moderators_name())
+        i.discourse.group_delete(i.group_members_name())
+        i.discourse.group_delete(i.group_moderated_members_name())
+        i.discourse.category_delete(i.name_get())
+        i.discourse.user_delete(moderator)
+        i.discourse.user_delete(owner)
+        i.discourse.user_delete(member)
+        i.discourse.user_delete(moderated_member)
+
+
 def test_importer_members_settings(test_options):
     i = importer.Importer(test_options)
     owner = 'owner@example.com'
@@ -397,39 +460,48 @@ def user_cannot_see_category(i, email):
     return True
 
 
-def verify_moderation_is_no_archive_is_public(i, nonmember, member, moderator):
+def verify_moderation_is_no_archive_is_public(
+        i, nonmember, member, moderated_member, moderator):
     subject, topic_id = email_stranger_ok(i, i.email_get())
     assert topic_id == user_can_see_topic(i, nonmember, subject)
 
 
-def verify_moderation_is_no_archive_is_private(i, nonmember, member, moderator):
+def verify_moderation_is_no_archive_is_private(
+        i, nonmember, member, moderated_member, moderator):
     subject, topic_id = email_stranger_ok(i, i.email_get())
     assert user_cannot_see_category(i, nonmember)
     assert topic_id == user_can_see_topic(i, member, subject)
+    topic_post_forbidden(i, moderated_member)
 
 
-def verify_moderation_is_ignore_archive_is_public(i, nonmember, member, moderator):
+def verify_moderation_is_ignore_archive_is_public(
+        i, nonmember, member, moderated_member, moderator):
     topic_post_forbidden(i, nonmember)
+    topic_post_forbidden(i, moderated_member)
     title, topic_id = topic_post_ok(i, member)
     email_stranger_ignored(i, i.email_get())
     assert topic_id == user_can_see_topic(i, nonmember, title)
 
 
-def verify_moderation_is_ignore_archive_is_private(i, nonmember, member, moderator):
+def verify_moderation_is_ignore_archive_is_private(
+        i, nonmember, member, moderated_member, moderator):
     assert user_cannot_see_category(i, nonmember)
     title, topic_id = topic_post_ok(i, member)
     assert topic_id == user_can_see_topic(i, member, title)
     email_stranger_ignored(i, i.email_get())
+    topic_post_forbidden(i, moderated_member)
 
 
-def verify_moderation_is_approval_archive_is_public(i, nonmember, member, moderator):
+def verify_moderation_is_approval_archive_is_public(
+        i, nonmember, member, moderated_member, moderator):
     title, topic_id = email_stranger_and_approve(i, i.email_get())
     assert topic_id == user_can_see_topic(i, nonmember, title)
     title, topic_id = topic_post_and_approve(i, member)
     assert topic_id == user_can_see_topic(i, nonmember, title)
 
 
-def verify_moderation_is_approval_archive_is_private(i, nonmember, member, moderator):
+def verify_moderation_is_approval_archive_is_private(
+        i, nonmember, member, moderated_member, moderator):
     assert user_cannot_see_category(i, nonmember)
     title, topic_id = email_stranger_and_approve(i, i.email_get())
     assert topic_id == user_can_see_topic(i, member, title)
@@ -457,11 +529,13 @@ def test_importer_category_settings(test_options, info, verify):
     moderator = 'moderator@example.com'
     member = 'member@example.com'
     nonmember = 'nonmember@example.com'
+    moderated_member = 'moderated_member@example.com'
     try:
         c = i.mailman.load_pickle()
         mailman_add(c, owner, owner=True)
         mailman_add(c, moderator, moderator=True)
         mailman_add(c, member)
+        mailman_add(c, moderated_member, user_options=Mailman.USER_OPTIONS_MODERATED)
         mailman_write_config(test_options.mailman_config, c)
         i.mailman.load()
         i.mailman.info.update(info)
@@ -473,7 +547,7 @@ def test_importer_category_settings(test_options, info, verify):
 
         _, _ = i.importer()
 
-        verify(i, nonmember, member, moderator)
+        verify(i, nonmember, member, moderated_member, moderator)
 
     finally:
         i.discourse.category_topics_delete(i.name_get())
@@ -484,6 +558,7 @@ def test_importer_category_settings(test_options, info, verify):
         i.discourse.user_delete(owner)
         i.discourse.user_delete(member)
         i.discourse.user_delete(nonmember)
+        i.discourse.user_delete(moderated_member)
 
 
 def test_importer_category_settings_force_private(test_options):
